/********************************************************************
 ** bip.h
 ** bip: namespace for binary and byte image processing
 ********************************************************************/

#if !defined(BIP_H)
#define BIP_H 1

#include "image.h"
#include "point.h"
#include "global.h"

namespace stdi {
  
  /** Features of a region, all the z-components are set to 0 in 2D */
  struct RegionFeature {
    
    RegionFeature(){ memset(this, 0, sizeof(RegionFeature)); }
    ~RegionFeature(){}
    
    int area() const { return (xmax-xmin+1)*(ymax-ymin+1); }
    int volume() const { return (xmax-xmin+1)*(ymax-ymin+1)*(zmax-zmin+1); }

    static vector<float> meanArray(const vector<RegionFeature> & );
    static vector<float> varArray(const vector<RegionFeature> & );
    static vector<int> npixArray(const vector<RegionFeature> & );
    
    int xmin, xmax, ymin, ymax, zmin, zmax; ///< range
    int xc, yc, zc, radius; ///< center of region
    int npix; ///< number of pixels(voxels)
    float mean, var; ///< statistical features

  };


  /** A "label" image is a SHORT image, background pixels are set to < 0 */
  namespace bip {

    /** Number of regions */
    int regionNumber(const ShortImage &);

    /** Is binary image? */
    bool isBinaryImage(const Image &I);

    /** Reverse a BYTE image */
    void reverse(ByteImage &I);

    /** Erode a binary image */
    void erode(ByteImage &I);
    /** Erode a short image */
    void erode(ShortImage &I);
    /** dilate a binary image */
    void dilate(ByteImage &I);

    /** Threshold image */
    ByteImage threshold(const Image &I, float thr);
    /** compute threshold for image */
    float computeThreshold(const Image &I, float scale); 

    /** Edge of a BYTE, SHORT or INT image */
    ByteImage edge(const Image &img);

    /** Connected regions(2D and 3D) */
    ShortImage regions(const ByteImage &I, int &nc,
		       int intensity=stdi::Foreground,
		       int radius=1, stdi::NeighborhoodType = stdi::Four);
    ShortImage regions(const ByteImage &I);   /** Provided for convenience */

    ShortImage regions(const ShortImage &I, int &nc);
    ShortImage regions(const ShortImage &I);   /** Provided for convenience */
    
    void relabel(ShortImage &I);

    /** Boundaries of regions */
    vector<PointArray> boundaries(const ShortImage &);

    /** Internal structures */
    void removeInternalStructures(ByteImage &img);
    
    /** Select regions */
    void selectRegions(ByteImage &img, const Image &br, float min_int, int min_sz);
    void selectRegionsBySize(ByteImage &img, int min_size, int max_size=-1);
    
    /** Distance transform */
    ByteImage chamfer(const Image &img, bool for_watershed=false);
    FloatImage fchamfer(const Image &img);
    
    /** Propagates the labels to unlabeled pixels.  
	\param max_dist the maximum distance for the propagation. 
    */
    void propagate(ShortImage &img, const float max_dist);

    /** Convex hull algorithm (2D and 3D) */
    ShortImage convexHull(const ByteImage &img, int N=8);
    void convexHull(ShortImage &img, int N=8);
    
    /** Region statistics and features */
    vector<RegionFeature> regionFeatures(const ShortImage &label, int nlabel);
    vector<RegionFeature> regionFeatures(const ShortImage &label, int nlabel, 
					 const Image &br); /* br is brightness image */
    vector<RegionFeature> regionFeatures(const ShortImage &label, const Image &br);  
    vector<RegionFeature> regionFeatures(const ShortImage &label);
    
    vector<FloatArray> regionIntensityArrays(const ShortImage &label, 
					     const Image &br, 
					     bool sorted=false);
    vector<FloatArray> regionIntensityArrays(const ShortImage &label, int nc, 
					     const Image &br,
					     bool sorted=false);
    
    /** histogram of a given image */
    vector<int> histogram(const ByteImage & img, 
			  int min_int=0, int max_int=255);
    
  }// End of namespace bip

} // End of namespace stdi

#endif // BIP_H


