/**********************************************************************
 ** image_io.h
 ** Namespace for image file I/O
 ** A "read" method returns a pointer to NEW image.  
 **********************************************************************/

#if !defined(IMAGEIO_H)
#define IMAGEIO_H 1

#include "image.h"
#include <fstream>
#include <string>
#define TIFFTAG_METAMORPH 33629

namespace stdi {

  namespace image_io {

    using namespace std;

  
    class IcsInfo {

    public:

      IcsInfo(const string &_base_dataset_name=string());
      IcsInfo(const IcsInfo &);
      IcsInfo & operator= (const IcsInfo &);
      virtual ~IcsInfo();

      long getUTCseconds() const;
      string channelName() const;
      string datasetName() const;

      void setWithCurrentTime();
      void setWithChannelInfo(const string& ex_freq, 
			      const string& exp_time,
			      char filenamechannelchar);

      void annotate(const string &a);
      void addProperty(const vector<string> &a);
      void addParagraph(const string &a);

      void setScope(const string &_scope);
      void setSample(const string &dish_type, const string &cell_type,
		     const string &media_name, float media_vol,
		     float media_con, const string &oil_type);

      friend ostream& operator<< (ostream &stream, const IcsInfo & info);



    protected:

      string base_dataset_name; // including pathname

      string history_date;
      string history_utc_seconds;
      string param_ex_filter;
      string param_exp_time;
      string channel;
      vector<string> other_properties;
      string other_text;

      string annotation;
      string scope;
      string sample;

    private:
      
      long _UTCseconds;

    };


    class MultiChannelIcsInfo {
      
    public:

      MultiChannelIcsInfo(const string &_base_dataset_name=string());
      MultiChannelIcsInfo(const MultiChannelIcsInfo &);
      MultiChannelIcsInfo & operator= (const MultiChannelIcsInfo &);
      virtual ~MultiChannelIcsInfo();

      long getUTCseconds() const;
      const vector<string> &channelNames() const;
      const string &datasetName() const;

      void setWithCurrentTime();
      void setWithChannelInfo(const string& ex_freq, 
			      const string& exp_time,
			      char filenamechannelchar);

      void annotate(const string &a);
      void addProperty(const vector<string> &a);

      void setScope(const string &_scope);
      void setSample(const string &dish_type, const string &cell_type,
		     const string &media_name, float media_vol,
		     float media_con, const string &oil_type);

      friend ostream& operator<< (ostream &stream, const MultiChannelIcsInfo & info);

  
    protected:

      string base_dataset_name; // including pathname

      string history_date;
      string history_utc_seconds;

      vector<string> param_ex_filter;
      vector<string> param_exp_time;
      vector<string> channel;

      vector<string> other_properties;
      string annotation;
      string scope;
      string sample;

    private:

      long _UTCseconds;

    };


    /** read image with formats(TIFF, IHT, PGM, ICS, UIF), recognized by extensions */
    Image *readImage(const string &filename);
    /** write image with formats(TIFF, IHT, PGM, ICS, UIF), recognized by extensions */
    void writeImage(const string &filename, const Image &img);
    
    /** read tiff */
    //Image *readTIFF(const string &filename, bool output_grayscale=true);
    /** read tiff */
    //Image* read8And16Tiff(const string & filename);
	
    /** read STK */
    //Image* readSTK(const string & filename); 
    /** read IHT */
    Image *readIHT(const string &filename);
    /** write IHT */
    void writeIHT(const string &filename, const Image &img);
    
    /** read ICS */
    Image *readICS(const string &filename);
    /** write ICS */
    void writeICS(const string &filename, const Image &img);
    /** write ICS with designated ics info*/
    void writeICS(const IcsInfo &info, const Image &img);
    /** write ICS with designated multi-channel ics info*/
    void writeICS(const MultiChannelIcsInfo &info, const Image &img);

    
    /** read UIF */
    Image *readUIF(const string &filename);
    /** write UIF */
    void writeUIF(const string &filename, const Image &img); 
    
    /** read PGM */
    Image *readPGM(const string &filename);
    
    /** write PGM */
    //void writePGM(const string &filename, const Image &img);

    /** write PPM */
    void writePPM(const string &filename, const ByteImage &img);
    
    /** convert to BYTE image */
    ByteImage convertToByteImage(const Image &img, bool forced_scaling=false);
    
    /** convert to SHORT image */
    ShortImage convertToShortImage(const Image &img);

    /** convert to INT image */
    IntImage convertToIntImage(const Image &img);
    
    /** Convert to FLOAT image */
    FloatImage convertToFloatImage(const Image &img);
    
    /* Convert to UInt image **/
    UIntImage convertToUIntImage(const Image &img); //sree
        
    

  } // End of namespace image_io

} // End of namespace stdi

#endif //   IMAGEIO_H

