/*
 Bahram Parvin(B_Parvin@lbl.gov), Qing Yang, Hang Chang(hchang@lbl.gov)
 
 This software library is a "simplified version" of the method described in

 Iterative Voting for Inference of Structural Saliency and Characterization of Subcellular Events
 Bahram Parvin, Qing Yang, Ju Han, Hang Chang, Bjorn Rydbery and Mary Helen Barcellos-Hoff.

 in IEEE TRANSACTIONS ON IMAGE PROCESSING

 Simplified version replaces double Gaussian kernels with uniform kernels to improve computational performance.


 Example usage:
 
 #include "circularvoting.h"
 #include "image_io.h"
 #include "image.h"
 #include "bip.h"
 #include "global.h"
 #include <string>
 
 using namespace std;
 using namespace stdi;
 using namespace voting;
 
 int main( const int argc, const char ** argv ){
 
   if( argc != 9 ){
     cerr<<"Invalid number of parameters"<<endl;
     cerr<<"Usage:\t image_file_name channel hmin hmax radius min_grad min_vote sigma\n";
     return 1;
   }
   string fn = argv[1];
   int channel = atoi(argv[2]);
   float hmin = atof(argv[3]);
   float hmax = atof(argv[4]);
   int radius = atoi(argv[5]);
   float min_grad = atof(argv[6]);
   float min_vote = atof(argv[7]);
   float sigma = atof(argv[8]);
  
   Image * img = NULL;
   img = image_io::readImage(fn.c_str());

   if( img == NULL ){
     cerr<<"Image is not valid"<<endl;
     return 1;
   }
  
   CircularVoting cv;
   cv.setParams(CircularVoting::BrightReg, hmin, hmax,radius, min_grad, min_vote, sigma, CircularVoting::PowMethod, (float) 1, false, true);
   cv.saveHistory(true);
   cv.setPrefix("tmp/");
   cv.compute(*(img->frame(channel)));
   PointArray seeds = cv.centers();
   vector<float> weights = cv.center_weights();

   int i;
   for( i = 0; i < seeds.size(); i ++ ){
     cout << "\nSeed#"<<i<<"\t("<<seeds[i].x()<<",\t"<<seeds[i].y()<<")\t";
     cout << "weights:\t"<<weights[i]<<endl;
   }
    
   delete img; img = NULL;

   return 0;
 }
 */

#ifndef CIRCULARVOTING_H
#define CIRCULARVOTING_H 

#include <image.h>
#include <point.h>
#include <global.h>

namespace stdi {
  
  namespace voting {
    
    /** Voting point */
    struct VPoint2D {
      short x, y;
      short xc, yc; //< center point 
      short angIndex; //< angle index
      int pos;
      float mag; //< magnitude
    };
    
    
    /** Weighted point */
    struct WPoint2D {    
      
      static int nx, ny;
      WPoint2D(int xx=0, int yy=0, float ww=0)
	: x(xx), y(yy), d(0), w(ww) {}
    
      static void setImageSize(int xsize, int ysize)
      {  
	nx = xsize;
	ny = ysize;
      }
      
      void setPos() 
      {  d = x + y*nx; }
      
      int x, y; //< coordinates
      int d; //< offset position
      float w; //< weight
    };

    /** An intersection of cone */
    //  typedef vector<WPoint2D> ConePlane2D;  
    struct ConePlane2D : public vector<WPoint2D> {
      
      static int nx, ny;
      
      ConePlane2D(int n, const value_type &v)
	: vector<WPoint2D>(n, v)
      { }
      
      ConePlane2D()
	: vector<WPoint2D>()
      { }
      
      static void setImageSize(int xsize, int ysize)
      {  
	nx = xsize;
	ny = ysize;
      }
      
      void setPos()
      {
	for(iterator it = begin(); it!=end(); it++)
	  it->setPos();     
      }
      
      
      float center_weight;
      
    };

    /** \brief A 2D cone */
    struct Cone2D : public vector<ConePlane2D> {

      static int nx, ny; //< image size
    
      Cone2D();
      void setDirection(float xx, float yy);
      static void setImageSize(int xsize, int ysize)
      {  
	nx = xsize;
	ny = ysize;
      }
      
      void init()
      {
	for(iterator it = begin(); it!=end(); it++) {
	  it->setPos();
	}
      }
      
      void vote(float* p, const VPoint2D& vp);
      void vote(float* p, float* pmask, const VPoint2D& vp);
      
      float dx, dy; //< direction

    };
  
    class CircularVoting {
    
    public:

      /** Region type */
      enum RegionType { BrightReg, DarkReg, AllReg };
      
      /** weighting method */
      enum WeightingMethod { PowMethod, ExpMethod };


      static const float epsilon;
      static const float pi;

      CircularVoting();
      ~CircularVoting();
      
      /**
	 Set parameters for circular voting 
	 \param reg_type region type
	 \param hmin lower bound of voting range, measured in pixel
	 \param hmax upper bound of voting range, measured in pixel, usually set as 3/4 of the object diameter
	 \param r aperture for voting, measured in pixel
	 \param min_grad gradient threshold for voting points, used for acceleration by eliminating voting points whose gradients are less than the threshold
	 \param threshold saliency threshold for picking up object centers after voting; In case abs_thresh == TRUE, this value is directly used; when abs_thresh == FALSE, this value should be within (0,1), and the real threshold will be computed implicitly
	 \param scale scale for gradient computation with DoG
	 \param weighting_method weighting method for gradient features
	 \param weighting_param weighting parameter
	 \param zc_only TRUE - voting points should also be the zero-crossing points of the image; FALSE - otherwise
	 \param abs_thresh TRUE - use input(threshold) directly; FALSE - use input(threshold) * max_voted_value as the threshold
       */
      void setParams(RegionType reg_type, 
		     int hmin, 
		     int hmax, 
		     int r,   
		     float min_grad,
		     float threshold, 
		     float scale=1.5,
		     WeightingMethod weighting_method=PowMethod,
		     float weighting_param=1,
		     bool zc_only=false,
		     bool abs_thresh = true);
      
      /** main user interface for voting */
      void compute(const Image& I);
      
      /**
	 \return image with voted centers and accumulated votes
      */
      FloatImage votingImage();
      
      /**
	 \return voted object centers
      */
      const PointArray& centers() const;
      
      /**
	 \return weights of voted object centers, these weights equal to accumulated votes
      */
      const vector<float>& center_weights() const;
      
            
      /**
	 Set the prefix of the path to store the voting landscope
      */
      void setPrefix(const string& );
      
      /**
	 Set whether to save voting landscope
      */
      void saveHistory(bool yn=true);

    private:
      
      /**
	 Compute voting aperture in the next iteration
	 \param h voting range
	 \param r current voting aperture
	 \return voting aperture in the next iteration
      */
      int nextConeRadius(int h, int r);
      
      /** voting */
      void vote();

      /** 
	  Compute cone for voting (2D cone is a triangle)
	  \param hmin lower bound of voting range, measured in pixel
	  \param hmax upper bound of voting range, measured in pixel, usually set as 3/4 of the object diameter
      */
      void computeCones(int hmin, int hmax, int radius);
      
      /**
	 Update voting direction for a given point: vp; the new direction is from vp to the point with maximum voting response within vp's voting range.
      */
      void getMaxDirection(VPoint2D& vp);
      
      /**
	 Update voting direction for a given point: vp; the new direction is from vp to the point (x,y) within vp's voting range, in which (x,y) is the average coordinate value weighted by its votes at current iteration.
      */
      void getMeanDirection(VPoint2D& vp);
      
      /**
	 \return The angle index in the look-up table
      */
      int computeAngleIndex(float dx, float dy) const;

      PointArray _centers; //< detected centers of objects
      vector<float> _center_weights; //< accumulated votes for each center
    
      FloatImage _sum, _mask, _vote; //<sum of voting

      vector<VPoint2D> _voting_points; //< voting candidate points
    
      vector<Cone2D> _cones; //< pre-computed cone structures for voting
      float delta_theta; //< step size of theta
      int ntheta; //< number of bins for 2*PI
      int nx, ny, npix; //< size of image
      int bw;//< padding size for border pixels during voting 


      float _min_grad; //< gradient threshold for voting candidate points
      int _hmin, _hmax, _radius; //< lower and upper bound for voting range
      RegionType _reg_type; //< region type
      float _threshold; //< threshold for accumulated votes
      float _scale; // scale for detecting gradient information via DoG

      WeightingMethod _weighting_method; // weighting method for gradient feature
      float _weighting_param; // weighting parameter

      bool _zc_only; //< whether only zero-crossing points can be voting points
      
      string _prefix; //< prefix for path to record voting landscope in each iteration

      bool _save_history; //< whether save voting landsope in each iteration
      
      bool _abs_thresh; //< use absolute value as threshold
    };

  } // voting
} // stdi

#endif 
